#include "dialog.h"
#include "ui_dialog.h"

#include <QProcess>
#include <QDirIterator>
#include <QFileDialog>
#include <QMessageBox>

#include <vcomment.h>

//QString vorbiscomment_path;

Dialog::Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog)
{
    ui->setupUi(this);
	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);

	ui->progressBar->setMinimum(0);
	ui->progressBar->setMaximum(1000);
	ui->progressBar->setValue(0);
}

Dialog::~Dialog()
{
    delete ui;
}

void Dialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

class MyProcess : public QProcess {
public:
	qint64 readData(char *data, qint64 maxlen)
	{
		return QProcess::readData(data, maxlen);
	}
};

struct aborted {
};

#include <QTextCodec>

void Dialog::invoke_vorbiscomment(QString commandline, std::vector<QString> *lines)
{
	QString text;
	MyProcess process;

	QTextCodec *codec = QTextCodec::codecForLocale();

	lines->clear();

//	QString cmd = "\"" + vorbiscomment_path + "\" " + commandline;
//	process.start(cmd);

	process.setReadChannel(QProcess::StandardOutput);

	while (1) {
		while (process.waitForReadyRead(0)) {
			char tmp[1000];
			long long n = process.readData(tmp, sizeof(tmp));
			if (n < 1) {
				break;
			}
//			text += QString::fromUtf8(tmp, (int)n);
			text += codec->toUnicode(tmp, n);
		}
		if (process.waitForFinished(100) || process.state() == QProcess::NotRunning) {
			break;
		}
	}
	while (1) {
		int i = text.indexOf('\r');
		if (i == -1) {
			i = text.indexOf('\n');
			if (i == -1) {
				break;
			}
			i++;
		} else {
			i++;
			if (text[i] == '\n') {
				i++;
			}
		}
		QString line = QString::fromUtf16(text.utf16(), i);
		lines->push_back(line);
		text = QString::fromUtf16(text.utf16() + i, text.size() - i);
	}
	if (!text.isEmpty()) {
		lines->push_back(text);
	}
}

static void split_path(QString const &path, QStringList *result)
{
	result->clear();
	wchar_t const *ptr = (wchar_t const *)path.utf16();
	wchar_t const *end = ptr + path.size();
	while (ptr < end) {
		wchar_t const *next = ptr;
		while (next < end) {
			if (*next == '/' || *next == '\\') {
				break;
			}
			next++;
		}
		result->append(QString::fromUtf16((ushort const *)ptr, next - ptr));
		if (next == end) {
			break;
		}
		ptr = next + 1;
	}
}

void Dialog::scan(QString dir, std::vector<item_t> *out)
{
	QDirIterator it(dir);
	while (it.hasNext()) {
		QString path = it.next();
		QFileInfo fi = it.fileInfo();
		if (!fi.fileName().startsWith('.')) {
			if (fi.isDir()) {
				scan(path, out);
				asm("nop");
			} else {
				if (path.endsWith(".ogg", Qt::CaseInsensitive)) {
					QStringList strlist;
					split_path(path, &strlist);
					QString artist, album;
					if (strlist.size() >= 3) {
						artist = strlist[strlist.size() - 3];
					}
					if (strlist.size() >= 2) {
						album = strlist[strlist.size() - 2];
					}
					QString basename = fi.completeBaseName();
					ushort const *p = basename.utf16();
					while (iswdigit(*p)) p++;
					while (iswspace(*p)) p++;
					QString title = QString::fromUtf16(p);
					item_t item;
					item.dir = dir;
					item.path = path;
					item.artist = artist;
					item.album = album;
					item.title = title;
					out->push_back(item);
				}
			}
		}
	}
}


struct vcomment_reader_container_t {
	std::map<QString, QString> *map;
};

struct vcomment_writer_container_t {
	std::map<QString, QString> const *map;
	std::map<QString, QString>::const_iterator it;
};


void __cdecl read_comment_handler(void *cookie, wchar_t const *k, wchar_t const *v)
{
	vcomment_reader_container_t *t = (vcomment_reader_container_t *)cookie;
	QString key = QString::fromUtf16((ushort const *)k);
	QString val = QString::fromUtf16((ushort const *)v);
	(*t->map)[key] = val;
}

void __cdecl write_comment_handler(void *cookie, wchar_t const **k, wchar_t const **v)
{
	vcomment_writer_container_t *t = (vcomment_writer_container_t *)cookie;
	if (t->it == t->map->end()) {
		*k = *v = 0;
		return;
	}
	*k = (wchar_t const *)t->it->first.utf16();
	*v = (wchar_t const *)t->it->second.utf16();
	t->it++;
}

bool read_comments(QString path, std::map<QString, QString> *map)
{
	vcomment_reader_container_t t;
	t.map = map;
	return read_vorbis_comments((wchar_t const *)path.utf16(), read_comment_handler, &t) != 0;
}

bool write_comments(QString path, std::map<QString, QString> const *map)
{
	vcomment_writer_container_t t;
	t.map = map;
	t.it = t.map->begin();
	return write_vorbis_comments((wchar_t const *)path.utf16(), write_comment_handler, &t) != 0;
}


QString lookup(std::map<QString, QString> const &map, QString const &key)
{
	std::map<QString, QString>::const_iterator it = map.find(key);
	if (it == map.end()) {
		return QString();
	}
	return it->second;
}

bool Dialog::edit(item_t const &item)
{
	QString cmd;
	std::vector<QString> lines;

	std::map<QString, QString> map;
	read_comments(item.path, &map);

	if (lookup(map, "ARTIST") == item.artist && lookup(map, "ALBUM") == item.album && lookup(map, "TITLE") == item.title) {
		return false;
	}

	map.clear();
	map["ARTIST"] = item.artist;
	map["ALBUM"] = item.album;
	map["TITLE"] = item.title;
	write_comments(item.path, &map);

	return true;
}


void Dialog::on_pushButton_Start_clicked()
{
	if (directory.isEmpty()) {
		QMessageBox::warning(this, this->windowTitle(), tr("Location not specified."));
		return;
	}
	ui->pushButton_Start->setEnabled(false);
	ui->pushButton_Abort->setEnabled(true);
	int count = 0;
	int skipped = 0;
	int changed = 0;
	try {
		interrupted = false;
		std::vector<item_t> items;
		scan(directory, &items);
		ui->label_total->setText(QString::number(items.size()));
		ui->progressBar->setMinimum(0);
		ui->progressBar->setMaximum(items.size());
		for (std::vector<item_t>::const_iterator it = items.begin(); it != items.end(); it++) {
			ui->label_dir->setText(it->dir);
			ui->label_name->setText(it->title);
			if (edit(*it)) {
				changed++;
			} else {
				skipped++;
			}
			count++;
			ui->label_changed->setText(QString::number(changed));
			ui->label_skipped->setText(QString::number(skipped));
			ui->progressBar->setValue(count);
			QCoreApplication::processEvents();
			if (interrupted) {
				throw aborted();
			}
		}
		asm("nop");
	} catch (aborted) {
	}
	ui->pushButton_Start->setEnabled(true);
	ui->pushButton_Abort->setEnabled(false);
	ui->progressBar->setValue(0);
	ui->label_dir->setText(directory);

	wchar_t tmp[1000];
	swprintf(tmp, (wchar_t const *)tr("%d files changed.").utf16(), changed);
	QMessageBox::information(this, this->windowTitle(), QString::fromUtf16((ushort const *)tmp));
}


void Dialog::on_pushButton_Abort_clicked()
{
	interrupted = true;
	ui->pushButton_Start->setEnabled(true);
	ui->pushButton_Abort->setEnabled(false);
}

void Dialog::on_pushButton_Location_clicked()
{
	QFileDialog::Options options = QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly;
	QString dir = QFileDialog::getExistingDirectory(this, tr("Select the folder."), directory, options);
	if (dir.isEmpty()) {
		return;
	}
	directory = dir;
	ui->label_dir->setText(directory);
}
