
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "utf8.h"
#include "vcedit.h"

#include <share.h>
#include <wchar.h>

//#include <string>
//#include <map>

#pragma warning(disable:4996)

#define BUILD_VCOMMENT_DLL
#include "vcomment.h"

typedef int bool;
enum {
	false = 0,
	true = 1,
};


static void get_comments(vorbis_comment const *vc, fn_read_comment fn, void *cookie)
{
	int i;

	//map->clear();

	for (i = 0; i < vc->comments; i++) {
		wchar_t *value = make_unicode_string((unsigned char const *)vc->user_comments[i]);
		wchar_t *e = wcschr(value, '=');
		if (e) {
			*e++ = 0;
			fn(cookie, value, e);
		}
		free(value);
	}
}

static void add_comments(vorbis_comment *vc, fn_write_comment fn, void *cookie)
{
	//for (std::map<std::wstring, std::wstring>::const_iterator it = map->begin(); it != map->end(); it++) {
	while (1) {
		wchar_t const *wk = 0;
		wchar_t const *wv = 0;
		unsigned char *k, *v;
		fn(cookie, &wk, &wv);
		if (!wk || !wv) {
			break;
		}
		k = make_utf8_string(wk);
		v = make_utf8_string(wv);
		vorbis_comment_add_tag(vc, (char const *)k, (char const *)v);
		free(k);
		free(v);
	}
}

bool VCOMMENT_DLL_EXPORT(read_vorbis_comments)(wchar_t const *path, fn_read_comment fn, void *cookie)
{
	bool success = false;
	FILE *fp = _wfsopen(path, L"rb", _SH_DENYNO);
	if (fp) {
		vcedit_state *state = vcedit_new_state();
		if (vcedit_open(state, fp) >= 0) {
			vorbis_comment *vc = vcedit_comments(state);
			get_comments(vc, fn, cookie);
			success = true;
		}
		vcedit_clear(state);
		fclose(fp);
	}
	return true;
}

bool VCOMMENT_DLL_EXPORT(write_vorbis_comments)(wchar_t const *path, fn_write_comment fn, void *cookie)
{
	bool success = false;
	FILE *in_fp;
	FILE *tmp_fp;

	wchar_t *tmppath = malloc(sizeof(wchar_t) * (wcslen(path) + 4));
	wcscpy(tmppath, path);
	wcscat(tmppath, L"tmp");

	in_fp = _wfsopen(path, L"rb", _SH_DENYNO);
	tmp_fp = _wfsopen(tmppath, L"wb", _SH_DENYRW);

	if (in_fp && tmp_fp) {

		vcedit_state *state = vcedit_new_state();

		if (vcedit_open(state, in_fp) >= 0) {
			vorbis_comment *vc = vcedit_comments(state);
			vorbis_comment_clear(vc);
			vorbis_comment_init(vc);

			add_comments(vc, fn, cookie);

			if (vcedit_write(state, tmp_fp) >= 0) {
				success = true;
			}
		}

		vcedit_clear(state);
	}

	if (in_fp) fclose(in_fp);
	if (tmp_fp) fclose(tmp_fp);

	if (success) {
		_wunlink(path);
		_wrename(tmppath, path);
	} else {
		_wunlink(tmppath);
	}

	free(tmppath);

	return success;
}

#if 0
/**********
   main.c

   This is the main function where options are read and written
   you should be able to just read this function and see how
   to call the vcedit routines. Details of how to pack/unpack the
   vorbis_comment structure itself are in the following two routines.
   The rest of the file is ui dressing so make the program minimally
   useful as a command line utility and can generally be ignored.

***********/

#include <io.h>

int main(int argc, char **argv)
{
	//vcedit_state *state;
	//vorbis_comment *vc;
	//param_t	*param;
	//int i;

	//setlocale(LC_ALL, "");
	//bindtextdomain(PACKAGE, LOCALEDIR);
	//textdomain(PACKAGE);


	/* take care of opening the requested files */
	/* relevent file pointers are returned in the param struct */

	/* which mode are we in? */

	std::wstring path = L"D:\\music\\VT\\sora no uta\\01 T[̃r[.ogg";

	if (0) {

		std::map<std::wstring, std::wstring> map;

		read_vorbis_comments(path.c_str(), &map);
		return 0;		
	}


	{

		std::map<std::wstring, std::wstring> map;
		map[L"ARTIST"] = L"hoge";
		map[L"ALBUM"] = L"fuga";
		map[L"TITLE"] = L"piyo";

		write_vorbis_comments(path.c_str(), &map);

	}
	return 0;
}

#endif
